@@ -1,12 +1,13 @@
 import enum
-from functools import reduce
-import requests
-import re
 from dataclasses import dataclass
+from functools import reduce
+from typing import Callable
+from urllib.parse import parse_qs, urlparse
 
-from typing import Callable, Generic, List, TypeVar, TypedDict, Union
+import requests
 
 from .config import Config
+from .interceptors.types import InterceptorHandler, Interceptors
 
 
 class HTTPMethods(enum.Enum):
@@ -24,8 +25,6 @@ class NoUrlException(Exception):
     NoUrlException represents a missing URL
     """
 
-    pass
-
 
 class ConnectionException(Exception):
     """
@@ -34,18 +33,14 @@ class ConnectionException(Exception):
     ConnectionException is not a 4xx response
     """
 
-    pass
-
 
 class AuthenticationException(Exception):
     """
     AuthenticationException represents a failure to authenticate
     """
 
-    pass
 
-
-def _get_dict(o: any):
+def _get_dict(o: any) -> dict:
     """
     get_dict attempts to convert objects to dict, nested
     """
@@ -63,40 +58,12 @@ def _get_dict(o: any):
         return o
 
 
-T = TypeVar("T")
-
-
-class InterceptorHandler(Generic[T]):
-    handlers: List[T]
-    "list containing the interceptors"
-
-    def __init__(self):
-        self.handlers = []
-
-    def use(self, interceptor: Union[T, List[T]]) -> None:
-        """
-        method helper for registering an interceptor
-        Parameters
-        ----------
-        interceptor: function interceptor for requests/responses
-        """
-        if isinstance(interceptor, list):
-            self.handlers = self.handlers + interceptor
-        else:
-            self.handlers.append(interceptor)
-
-
 RequestInterceptor = Callable[[requests.Request], requests.Request]
 """ REST Request interceptor """
 ResponseInterceptor = Callable[[requests.Response], requests.Response]
 """ REST Request interceptor """
 
 
-class Interceptors(TypedDict):
-    response: InterceptorHandler[ResponseInterceptor]
-    request: InterceptorHandler[RequestInterceptor]
-
-
 @dataclass
 class RestClient:
     """
@@ -105,7 +72,7 @@ class RestClient:
 
     config: Config
     base_url: str
-    _interceptors: Interceptors = None
+    _interceptors: Interceptors[ResponseInterceptor, RequestInterceptor] = None
 
     def __init__(self, config) -> None:
         self.config = config
@@ -198,15 +165,15 @@ class RestClient:
                 if yield_count == max_items:
                     return
             try:
-                curr_params["next"] = re.sub(
-                    r".*next=", "", response["links"]["next"]["href"]
+                curr_params["next"] = get_next_query_value(
+                    response["links"]["next"]["href"]
                 )
             except:
                 done = True
 
 
 class RestClientInstance:
-    interceptors: Interceptors
+    interceptors: Interceptors[ResponseInterceptor, RequestInterceptor]
 
     def __init__(self, restClient: RestClient) -> None:
         self._restClient = restClient
@@ -215,3 +182,7 @@ class RestClientInstance:
 
     def __call__(self, **all: requests.Request) -> requests.Request:
         return self._restClient.rest(**all)
+
+
+def get_next_query_value(url_str: str) -> str:
+    return parse_qs(urlparse(url_str).query)["next"][0]
